From d0f418d2b1728ef729e53f1300086ca8b3d5e217 Mon Sep 17 00:00:00 2001
From: Steve Lhomme <robux4@videolabs.io>
Date: Mon, 4 Dec 2017 11:14:50 +0100
Subject: [PATCH] [RFC] fix demux_sys_t ODR for C++ modules

Here's an example of how to fix #17078 without reverting to the unsafe "void".
---
 include/vlc_demux.h        |  4 ++++
 modules/access/live555.cpp | 45 ++++++++++++++++++++++++---------------------
 2 files changed, 28 insertions(+), 21 deletions(-)

diff --git a/include/vlc_demux.h b/include/vlc_demux.h
index f359f54..2646460 100644
--- a/include/vlc_demux.h
+++ b/include/vlc_demux.h
@@ -91,7 +91,11 @@ struct demux_t
         int          i_title;       /* idem, start from 0 (could be menu) */
         int          i_seekpoint;   /* idem, start from 0 */
     } info;
+#ifdef __cplusplus
+    DEMUX_SYS_T  *p_sys;
+#else
     demux_sys_t *p_sys;
+#endif
 
     /* Weak link to parent input */
     input_thread_t *p_input;
diff --git a/modules/access/live555.cpp b/modules/access/live555.cpp
index e5d9e73..62d6445 100644
--- a/modules/access/live555.cpp
+++ b/modules/access/live555.cpp
@@ -34,6 +34,9 @@
 
 #include <inttypes.h>
 
+#define DEMUX_SYS_T  sys_live555_t
+typedef struct sys_live555_t sys_live555_t;
+
 #include <vlc_common.h>
 #include <vlc_plugin.h>
 #include <vlc_input.h>
@@ -189,7 +192,7 @@ typedef struct
 
 class RTSPClientVlc;
 
-struct demux_sys_t
+struct sys_live555_t
 {
     char            *p_sdp;    /* XXX mallocated */
     char            *psz_pl_url; /* password-less URL */
@@ -243,7 +246,7 @@ class RTSPClientVlc : public RTSPClient
 public:
     RTSPClientVlc( UsageEnvironment& env, char const* rtspURL, int verbosityLevel,
                    char const* applicationName, portNumBits tunnelOverHTTPPortNum,
-                   demux_sys_t *p_sys) :
+                   sys_live555_t *p_sys) :
                    RTSPClient( env, rtspURL, verbosityLevel, applicationName,
                    tunnelOverHTTPPortNum
 #if LIVEMEDIA_LIBRARY_VERSION_INT >= 1373932800
@@ -253,7 +256,7 @@ public:
     {
         this->p_sys = p_sys;
     }
-    demux_sys_t *p_sys;
+    sys_live555_t *p_sys;
 };
 
 static int Demux  ( demux_t * );
@@ -289,7 +292,7 @@ static char *passwordLessURL( vlc_url_t *url );
 static int  Open ( vlc_object_t *p_this )
 {
     demux_t     *p_demux = (demux_t*)p_this;
-    demux_sys_t *p_sys = NULL;
+    sys_live555_t *p_sys = NULL;
 
     char *psz_url;
     int i_return;
@@ -328,7 +331,7 @@ static int  Open ( vlc_object_t *p_this )
 
     p_demux->pf_demux  = Demux;
     p_demux->pf_control= Control;
-    p_demux->p_sys     = p_sys = (demux_sys_t*)calloc( 1, sizeof( demux_sys_t ) );
+    p_demux->p_sys     = p_sys = (sys_live555_t*)calloc( 1, sizeof( sys_live555_t ) );
     if( !p_sys ) return VLC_ENOMEM;
 
     if( vlc_timer_create(&p_sys->timer, TimeoutPrevention, p_demux) )
@@ -461,7 +464,7 @@ error:
 static void Close( vlc_object_t *p_this )
 {
     demux_t *p_demux = (demux_t*)p_this;
-    demux_sys_t *p_sys = p_demux->p_sys;
+    sys_live555_t *p_sys = p_demux->p_sys;
 
     vlc_timer_destroy(p_sys->timer);
 
@@ -498,7 +501,7 @@ static inline Boolean toBool( bool b ) { return b?True:False; } // silly, no?
 static void default_live555_callback( RTSPClient* client, int result_code, char* result_string )
 {
     RTSPClientVlc *client_vlc = static_cast<RTSPClientVlc *> ( client );
-    demux_sys_t *p_sys = client_vlc->p_sys;
+    sys_live555_t *p_sys = client_vlc->p_sys;
     delete []result_string;
     p_sys->i_live555_ret = result_code;
     p_sys->b_error = p_sys->i_live555_ret != 0;
@@ -509,7 +512,7 @@ static void default_live555_callback( RTSPClient* client, int result_code, char*
 static bool wait_Live555_response( demux_t *p_demux, int i_timeout = 0 /* ms */ )
 {
     TaskToken task;
-    demux_sys_t * p_sys = p_demux->p_sys;
+    sys_live555_t * p_sys = p_demux->p_sys;
     p_sys->event_rtsp = 0;
     if( i_timeout > 0 )
     {
@@ -535,7 +538,7 @@ static void continueAfterDESCRIBE( RTSPClient* client, int result_code,
                                    char* result_string )
 {
     RTSPClientVlc *client_vlc = static_cast<RTSPClientVlc *> ( client );
-    demux_sys_t *p_sys = client_vlc->p_sys;
+    sys_live555_t *p_sys = client_vlc->p_sys;
     p_sys->i_live555_ret = result_code;
     if ( result_code == 0 )
     {
@@ -558,7 +561,7 @@ static void continueAfterOPTIONS( RTSPClient* client, int result_code,
                                   char* result_string )
 {
     RTSPClientVlc *client_vlc = static_cast<RTSPClientVlc *> (client);
-    demux_sys_t *p_sys = client_vlc->p_sys;
+    sys_live555_t *p_sys = client_vlc->p_sys;
     p_sys->b_get_param =
       // If OPTIONS fails, assume GET_PARAMETER is not supported but
       // still continue on with the stream.  Some servers (foscam)
@@ -575,7 +578,7 @@ static void continueAfterOPTIONS( RTSPClient* client, int result_code,
  *****************************************************************************/
 static int Connect( demux_t *p_demux )
 {
-    demux_sys_t *p_sys = p_demux->p_sys;
+    sys_live555_t *p_sys = p_demux->p_sys;
     Authenticator authenticator;
     vlc_credential credential;
     const char *psz_user = NULL;
@@ -690,7 +693,7 @@ bailout:
  *****************************************************************************/
 static int SessionsSetup( demux_t *p_demux )
 {
-    demux_sys_t             *p_sys  = p_demux->p_sys;
+    sys_live555_t             *p_sys  = p_demux->p_sys;
     MediaSubsessionIterator *iter   = NULL;
     MediaSubsession         *sub    = NULL;
 
@@ -1231,7 +1234,7 @@ static int SessionsSetup( demux_t *p_demux )
  *****************************************************************************/
 static int Play( demux_t *p_demux )
 {
-    demux_sys_t *p_sys = p_demux->p_sys;
+    sys_live555_t *p_sys = p_demux->p_sys;
 
     if( p_sys->rtsp )
     {
@@ -1292,7 +1295,7 @@ static bool HasSharedSession( MediaSubsession *session )
  *****************************************************************************/
 static void ResumeTrack( demux_t *p_demux, live_track_t *tk )
 {
-    demux_sys_t    *p_sys = p_demux->p_sys;
+    sys_live555_t    *p_sys = p_demux->p_sys;
 
     bool b_rtsp_tcp = var_GetBool( p_demux, "rtsp-tcp" ) ||
             var_GetBool( p_demux, "rtsp-http" );
@@ -1327,7 +1330,7 @@ static void ResumeTrack( demux_t *p_demux, live_track_t *tk )
  *****************************************************************************/
 static int Demux( demux_t *p_demux )
 {
-    demux_sys_t    *p_sys = p_demux->p_sys;
+    sys_live555_t    *p_sys = p_demux->p_sys;
     TaskToken      task;
 
     bool            b_send_pcr = true;
@@ -1486,7 +1489,7 @@ static int Demux( demux_t *p_demux )
  *****************************************************************************/
 static int Control( demux_t *p_demux, int i_query, va_list args )
 {
-    demux_sys_t *p_sys = p_demux->p_sys;
+    sys_live555_t *p_sys = p_demux->p_sys;
     int64_t *pi64, i64;
     double  *pf, f;
     bool *pb;
@@ -1751,7 +1754,7 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
  *****************************************************************************/
 static int RollOverTcp( demux_t *p_demux )
 {
-    demux_sys_t *p_sys = p_demux->p_sys;
+    sys_live555_t *p_sys = p_demux->p_sys;
     int i, i_return;
 
     var_SetBool( p_demux, "rtsp-tcp", true );
@@ -1906,7 +1909,7 @@ static void StreamRead( void *p_private, unsigned int i_size,
 
     live_track_t   *tk = (live_track_t*)p_private;
     demux_t        *p_demux = tk->p_demux;
-    demux_sys_t    *p_sys = p_demux->p_sys;
+    sys_live555_t    *p_sys = p_demux->p_sys;
     block_t        *p_block;
 
     //msg_Dbg( p_demux, "pts: %d", pts.tv_sec );
@@ -2151,7 +2154,7 @@ static void StreamClose( void *p_private )
 {
     live_track_t   *tk = (live_track_t*)p_private;
     demux_t        *p_demux = tk->p_demux;
-    demux_sys_t    *p_sys = p_demux->p_sys;
+    sys_live555_t    *p_sys = p_demux->p_sys;
     tk->state = live_track_t::STATE_IGNORED;
     p_sys->event_rtsp = 0xff;
     p_sys->event_data = 0xff;
@@ -2198,7 +2201,7 @@ static void TaskInterruptData( void *p_private )
 static void TimeoutPrevention( void *p_data )
 {
     demux_t *p_demux = (demux_t *) p_data;
-    demux_sys_t *p_sys = p_demux->p_sys;
+    sys_live555_t *p_sys = p_demux->p_sys;
     char *bye = NULL;
 
     /* Protect Live555 from us calling their functions simultaneously
@@ -2241,7 +2244,7 @@ static void TimeoutPrevention( void *p_data )
  *****************************************************************************/
 static int ParseASF( demux_t *p_demux )
 {
-    demux_sys_t    *p_sys = p_demux->p_sys;
+    sys_live555_t    *p_sys = p_demux->p_sys;
 
     const char *psz_marker = "a=pgmpu:data:application/vnd.ms.wms-hdr.asfv1;base64,";
     char *psz_asf = strcasestr( p_sys->p_sdp, psz_marker );
-- 
2.10.1.windows.1

